@@ -21,7 +21,12 @@ FIX_FP_TABLE = maketrans('de', 'DE')
 FIX_FP_TABLE2 = maketrans('dD', 'eE')
 
 
-BLANK_CARD = ' ' * 80
+CARD_LENGTH = 80
+BLANK_CARD = ' ' * CARD_LENGTH
+KEYWORD_LENGTH = 8  # The max length for FITS-standard keywords
+
+VALUE_INDICATOR = '= '  # The standard FITS value indicator
+HIERARCH_VALUE_INDICATOR = '='  # HIERARCH cards may use a shortened indicator
 
 
 class Undefined:
@@ -226,11 +231,11 @@ class CardList(list):
 
     @deprecated('3.1', alternative=':meth:`Header.index`')
     def index(self, card):
-        return self._header.cards.index(card)
+        return self._header._cards.index(card)
 
     @deprecated('3.1', alternative=':meth:`Header.count`')
     def count(self, card):
-        return self._header.cards.count(card)
+        return self._header._cards.count(card)
 
     @deprecated('3.1', alternative=':meth:`Header.index`', pending=False)
     def index_of(self, key, backward=False):
@@ -296,12 +301,13 @@ class CardList(list):
 
 
 class Card(_Verify):
-    length = 80
+    length = CARD_LENGTH
 
     # String for a FITS standard compliant (FSC) keyword.
-    _keywd_FSC_RE = re.compile(r'^[A-Z0-9_-]{0,8}$')
+    _keywd_FSC_RE = re.compile(r'^[A-Z0-9_-]{0,%d}$' % KEYWORD_LENGTH)
     # This will match any printable ASCII character excluding '='
-    _keywd_hierarch_RE = re.compile(r'^(?:HIERARCH )?(?:^[ -<>-~]+ ?)+$', re.I)
+    _keywd_hierarch_RE = re.compile(r'^(?:HIERARCH +)?(?:^[ -<>-~]+ ?)+$',
+                                    re.I)
 
     # A number sub-string, either an integer or a float in fixed or
     # scientific notation.  One for FSC and one for non-FSC (NFSC) format:
@@ -379,13 +385,13 @@ class Card(_Verify):
     _rvkc_identifier = r'[a-zA-Z_]\w*'
     _rvkc_field = _rvkc_identifier + r'(\.\d+)?'
     _rvkc_field_specifier_s = r'%s(\.%s)*' % ((_rvkc_field,) * 2)
-    _rvkc_field_specifier_val = (r'(?P<keyword>%s): (?P<val>%s\s*)' %
+    _rvkc_field_specifier_val = (r'(?P<keyword>%s): (?P<val>%s)' %
                                  (_rvkc_field_specifier_s, _numr_FSC))
-    _rvkc_keyword_val = r'\'%s\'' % _rvkc_field_specifier_val
+    _rvkc_keyword_val = r'\'(?P<rawval>%s)\'' % _rvkc_field_specifier_val
     _rvkc_keyword_val_comm = (r' +%s *(/ *(?P<comm>[ -~]*))?$' %
                               _rvkc_keyword_val)
 
-    _rvkc_field_specifier_val_RE = re.compile(_rvkc_field_specifier_val)
+    _rvkc_field_specifier_val_RE = re.compile(_rvkc_field_specifier_val + '$')
 
     # regular expression to extract the key and the field specifier from a
     # string that is being used to index into a card list that contains
@@ -401,6 +407,10 @@ class Card(_Verify):
 
     _commentary_keywords = ['', 'COMMENT', 'HISTORY', 'END']
 
+    # The default value indicator; may be changed if required by a convention
+    # (namely HIERARCH cards)
+    _value_indicator = VALUE_INDICATOR
+
     def __init__(self, keyword=None, value=None, comment=None, **kwargs):
         # For backwards compatibility, support the 'key' keyword argument:
         if keyword is None and 'key' in kwargs:
@@ -409,6 +419,7 @@ class Card(_Verify):
         self._keyword = None
         self._value = None
         self._comment = None
+        self._rawvalue = None
         self._image = None
 
         # This attribute is set to False when creating the card from a card
@@ -476,7 +487,8 @@ class Card(_Verify):
             # should be strictly disallowed.
             keyword = keyword.rstrip()
             keyword_upper = keyword.upper()
-            if len(keyword) <= 8 and self._keywd_FSC_RE.match(keyword_upper):
+            if (len(keyword) <= KEYWORD_LENGTH and
+                self._keywd_FSC_RE.match(keyword_upper)):
                 # For keywords with length > 8 they will be HIERARCH cards,
                 # and can have arbitrary case keywords
                 if keyword_upper == 'END':
@@ -490,11 +502,12 @@ class Card(_Verify):
                 # the old behavior makes it possible to create HEIRARCH cards
                 # that would otherwise be recognized as RVKCs
                 self._hierarch = True
+                self._value_indicator = HIERARCH_VALUE_INDICATOR
 
                 if keyword_upper[:9] == 'HIERARCH ':
                     # The user explicitly asked for a HIERARCH card, so don't
                     # bug them about it...
-                    keyword = keyword[9:]
+                    keyword = keyword[9:].strip()
                 else:
                     # We'll gladly create a HIERARCH card, but a warning is
                     # also displayed
@@ -593,6 +606,12 @@ class Card(_Verify):
             raise AttributeError('Values cannot be deleted from record-valued '
                                  'keyword cards')
 
+    @property
+    def rawvalue(self):
+        if self._rawvalue is None and self.value:
+            self._rawvalue = self.value
+        return self._rawvalue
+
     @property
     def comment(self):
         """Get the comment attribute from the card image if not already set."""
@@ -771,6 +790,8 @@ class Card(_Verify):
             if eq_idx < 0 or eq_idx > 9:
                 return False
             keyword, rest = image.split('=', 1)
+            if keyword in self._commentary_keywords:
+                return False
             match = self._rvkc_keyword_val_comm_RE.match(rest)
             if match:
                 field_specifier = match.group('keyword')
@@ -778,11 +799,14 @@ class Card(_Verify):
                                           field_specifier))
                 self._field_specifier = field_specifier
                 self._value = _int_or_float(match.group('val'))
+                self._rawvalue = match.group('rawval')
                 return True
         elif len(args) == 2:
             keyword, value = args
             if not isinstance(keyword, basestring):
                 return False
+            if keyword in self._commentary_keywords:
+                return False
             match = self._rvkc_keyword_name_RE.match(keyword)
             if match and isinstance(value, (int, float)):
                 field_specifier = match.group('field_specifier')
@@ -799,19 +823,20 @@ class Card(_Verify):
                                               field_specifier))
                     self._field_specifier = field_specifier
                     self._value = _int_or_float(match.group('val'))
+                    self._rawvalue = value
                     return True
 
     def _parse_keyword(self):
         if self._check_if_rvkc(self._image):
             return self._keyword
 
-        keyword = self._image[:8].strip()
+        keyword = self._image[:KEYWORD_LENGTH].strip()
         keyword_upper = keyword.upper()
-        value_indicator = self._image.find('= ')
+        value_indicator = self._image.find(VALUE_INDICATOR)
 
         special = self._commentary_keywords + ['CONTINUE']
 
-        if keyword_upper in special or 0 <= value_indicator <= 8:
+        if keyword_upper in special or 0 <= value_indicator <= KEYWORD_LENGTH:
             # The value indicator should appear in byte 8, but we are flexible
             # and allow this to be fixed
             if value_indicator >= 0:
@@ -822,12 +847,14 @@ class Card(_Verify):
                 self._modified = True
             return keyword_upper
         elif (keyword_upper == 'HIERARCH' and self._image[8] == ' ' and
-              '=' in self._image):
+              HIERARCH_VALUE_INDICATOR in self._image):
             # This is valid HIERARCH card as described by the HIERARCH keyword
             # convention:
             # http://fits.gsfc.nasa.gov/registry/hierarch_keyword.html
             self._hierarch = True
-            return self._image.split('=', 1)[0][9:].rstrip()
+            self._value_indicator = HIERARCH_VALUE_INDICATOR
+            keyword = self._image.split(HIERARCH_VALUE_INDICATOR, 1)[0][9:]
+            return keyword.strip()
         else:
             warnings.warn('The following header keyword is invalid or follows '
                           'an unrecognized non-standard convention:\n%s' %
@@ -841,7 +868,7 @@ class Card(_Verify):
         # for commentary cards, no need to parse further
         # Likewise for invalid cards
         if self.keyword.upper() in self._commentary_keywords or self._invalid:
-            return self._image[8:].rstrip()
+            return self._image[KEYWORD_LENGTH:].rstrip()
 
         if self._check_if_rvkc(self._image):
             return self._value
@@ -942,20 +969,20 @@ class Card(_Verify):
             keyword, valuecomment = image.split(' ', 1)
         else:
             try:
-                delim_index = image.index('= ')
+                delim_index = image.index(self._value_indicator)
             except ValueError:
                 delim_index = None
 
             # The equal sign may not be any higher than column 10; anything
             # past that must be considered part of the card value
             if delim_index is None:
-                keyword = image[:8]
-                valuecomment = image[8:]
+                keyword = image[:KEYWORD_LENGTH]
+                valuecomment = image[KEYWORD_LENGTH:]
             elif delim_index > 10 and image[:9] != 'HIERARCH ':
                 keyword = image[:8]
                 valuecomment = image[8:]
             else:
-                keyword, valuecomment = image.split('= ', 1)
+                keyword, valuecomment = image.split(self._value_indicator, 1)
         return keyword.strip(), valuecomment.strip()
 
     def _fix_keyword(self):
@@ -1009,13 +1036,13 @@ class Card(_Verify):
     def _format_keyword(self):
         if self.keyword:
             if self.field_specifier:
-                return '%-8s' % self.keyword.split('.', 1)[0]
+                return '%-*s' % (KEYWORD_LENGTH, self.keyword.split('.', 1)[0])
             elif self._hierarch:
                 return 'HIERARCH %s ' % self.keyword
             else:
-                return '%-8s' % self.keyword
+                return '%-*s' % (KEYWORD_LENGTH, self.keyword)
         else:
-            return ' ' * 8
+            return ' ' * KEYWORD_LENGTH
 
     def _format_value(self):
         # value string
@@ -1042,7 +1069,7 @@ class Card(_Verify):
             value = _format_value(value)
 
         # For HIERARCH cards the value should be shortened to conserve space
-        if not self.field_specifier and len(self.keyword) > 8:
+        if not self.field_specifier and len(self.keyword) > KEYWORD_LENGTH:
             value = value.strip()
 
         return value
@@ -1064,7 +1091,9 @@ class Card(_Verify):
             comment = self._format_comment()
 
         # equal sign string
-        delimiter = '= '
+        # by default use the standard value indicator even for HIERARCH cards;
+        # later we may abbreviate it if necessary
+        delimiter = VALUE_INDICATOR
         if is_commentary:
             delimiter = ''
 
@@ -1091,7 +1120,7 @@ class Card(_Verify):
             # longstring case (CONTINUE card)
             # try not to use CONTINUE if the string value can fit in one line.
             # Instead, just truncate the comment
-            if (isinstance(self.value, str) and
+            if (isinstance(self.value, basestring) and
                 len(value) > (self.length - 10)):
                 output = self._format_long_image()
             else:
@@ -1122,7 +1151,7 @@ class Card(_Verify):
         words = _words_group(value, value_length)
         for idx, word in enumerate(words):
             if idx == 0:
-                headstr = '%-8s= ' % self.keyword
+                headstr = '%-*s= ' % (KEYWORD_LENGTH, self.keyword)
             else:
                 headstr = 'CONTINUE  '
             value = value_format % word
@@ -1145,7 +1174,7 @@ class Card(_Verify):
         same type.
         """
 
-        maxlen = Card.length - len(self.keyword) - 1
+        maxlen = Card.length - KEYWORD_LENGTH
         value = self._format_value()
         output = []
         idx = 0
@@ -1167,7 +1196,7 @@ class Card(_Verify):
 
         # verify the equal sign position
         if (self.keyword not in self._commentary_keywords and
-            (self._image and self._image[:8].upper() != 'HIERARCH' and
+            (self._image and self._image[:9].upper() != 'HIERARCH ' and
              self._image.find('=') != 8)):
             errs.append(self.run_option(
                 option,
@@ -1179,7 +1208,8 @@ class Card(_Verify):
         # verify the key, it is never fixable
         # always fix silently the case where "=" is before column 9,
         # since there is no way to communicate back to the _keys.
-        if self._image and self._image[:8].upper() == 'HIERARCH':
+        if ((self._image and self._image[:8].upper() == 'HIERARCH') or
+                self._hierarch):
             pass
         else:
             keyword = self.keyword
