@@ -61,13 +61,15 @@ from pyfits.file import PYTHON_MODES, _File
 from pyfits.hdu.base import _BaseHDU, _ValidHDU
 from pyfits.hdu.hdulist import fitsopen
 from pyfits.hdu.image import PrimaryHDU, ImageHDU
-from pyfits.hdu.table import BinTableHDU, _TableBaseHDU
+from pyfits.hdu.table import BinTableHDU
 from pyfits.header import Header
-from pyfits.util import fileobj_closed, fileobj_name, isfile, _is_int
+from pyfits.util import (deprecated, fileobj_closed, fileobj_name, isfile,
+                         _is_int)
 
 
 __all__ = ['getheader', 'getdata', 'getval', 'setval', 'delval', 'writeto',
-           'append', 'update', 'info', 'tdump', 'tcreate']
+           'append', 'update', 'info', 'tdump', 'tcreate', 'tabledump',
+           'tableload']
 
 
 def getheader(filename, *args, **kwargs):
@@ -307,6 +309,9 @@ def setval(filename, keyword, *args, **kwargs):
         unmodified header.
     """
 
+    if 'do_not_scale_image_data' not in kwargs:
+        kwargs['do_not_scale_image_data'] = True
+
     value = kwargs.pop('value', None)
     comment = kwargs.pop('comment', None)
     before = kwargs.pop('before', None)
@@ -314,15 +319,9 @@ def setval(filename, keyword, *args, **kwargs):
     savecomment = kwargs.pop('savecomment', False)
 
     hdulist, extidx = _getext(filename, 'update', *args, **kwargs)
-    hdulist[extidx].header.update(keyword, value, comment, before, after,
-                                  savecomment)
-
-    # Ensure that data will not be scaled when the file is closed
-
-    for hdu in hdulist:
-       hdu._bscale = 1
-       hdu._bzero = 0
-
+    if keyword in hdulist[extidx].header and savecomment:
+        comment = None
+    hdulist[extidx].header.set(keyword, value, comment, before, after)
     hdulist.close()
 
 
@@ -352,15 +351,11 @@ def delval(filename, keyword, *args, **kwargs):
         unmodified header.
     """
 
+    if 'do_not_scale_image_data' not in kwargs:
+        kwargs['do_not_scale_image_data'] = True
+
     hdulist, extidx = _getext(filename, 'update', *args, **kwargs)
     del hdulist[extidx].header[keyword]
-
-    # Ensure that data will not be scaled when the file is closed
-
-    for hdu in hdulist:
-       hdu._bscale = 1
-       hdu._bzero = 0
-
     hdulist.close()
 
 
@@ -397,7 +392,7 @@ def writeto(filename, data, header=None, output_verify='exception',
     """
 
     hdu = _makehdu(data, header)
-    if not isinstance(hdu, PrimaryHDU) and not isinstance(hdu, _TableBaseHDU):
+    if hdu.is_image and not isinstance(hdu, PrimaryHDU):
         hdu = PrimaryHDU(data, header=header)
     hdu.writeto(filename, clobber=clobber, output_verify=output_verify,
                 checksum=checksum)
@@ -457,15 +452,13 @@ def append(filename, data, header=None, checksum=False, verify=True, **kwargs):
             f = fitsopen(filename, mode='append')
             f.append(hdu)
 
-            # Set a flag in the HDU so that only this HDU gets a checksum
-            # when writing the file.
+            # Set a flag in the HDU so that only this HDU gets a checksum when
+            # writing the file.
             hdu._output_checksum = checksum
             f.close(closed=closed)
         else:
             f = _File(filename, mode='append')
             hdu._output_checksum = checksum
-            # TODO: Fix this once an API for writing an HDU to a file is
-            # settled on
             hdu._writeto(f)
             f.close()
 
@@ -541,10 +534,9 @@ def info(filename, output=None, **kwargs):
         FITS file to obtain info from.  If opened, mode must be one of
         the following: rb, rb+, or ab+.
 
-    output : file, bool (optional)
-        A file-like object to write the output to.  If ``False``, does not
-        output to a file and instead returns a list of tuples representing the
-        HDU info.  Writes to ``sys.stdout`` by default.
+    output : file (optional)
+        File-like object to output the HDU info to.  Outputs to stdout by
+        default.
 
     kwargs
         Any additional keyword arguments to be passed to `pyfits.open`.
@@ -565,8 +557,8 @@ def info(filename, output=None, **kwargs):
     return ret
 
 
-def tdump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
-          clobber=False):
+def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
+              clobber=False):
     """
     Dump a table HDU to a file in ASCII format.  The table may be
     dumped in three separate files, one containing column definitions,
@@ -599,7 +591,7 @@ def tdump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
 
     Notes
     -----
-    The primary use for the `tdump` function is to allow editing in a
+    The primary use for the `tabledump` function is to allow editing in a
     standard text editor of the table data and parameters.  The
     `tcreate` function can be used to reassemble the table from the
     three ASCII files.
@@ -619,14 +611,20 @@ def tdump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
         datafile = root + '_' + repr(ext) + '.txt'
 
     # Dump the data from the HDU to the files
-    f[ext].tdump(datafile, cdfile, hfile, clobber)
+    f[ext].dump(datafile, cdfile, hfile, clobber)
 
     if closed:
         f.close()
-tdump.__doc__ += BinTableHDU.tdump_file_format.replace('\n', '\n    ')
+tabledump.__doc__ += BinTableHDU._tdump_file_format.replace('\n', '\n    ')
 
 
-def tcreate(datafile, cdfile, hfile=None):
+@deprecated('3.1', alternative=':func:`tabledump`')
+def tdump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
+          clobber=False):
+    tabledump(filename, datafile, cdfile, hfile, ext, clobber)
+
+
+def tableload(datafile, cdfile, hfile=None):
     """
     Create a table from the input ASCII files.  The input is from up
     to three separate files, one containing column definitions, one
@@ -652,14 +650,19 @@ def tcreate(datafile, cdfile, hfile=None):
 
     Notes
     -----
-    The primary use for the `tcreate` function is to allow the input of
+    The primary use for the `tableload` function is to allow the input of
     ASCII data that was edited in a standard text editor of the table
-    data and parameters.  The tdump function can be used to create the
+    data and parameters.  The tabledump function can be used to create the
     initial ASCII files.
     """
 
-    return BinTableHDU.tcreate(datafile, cdfile, hfile, replace=True)
-tcreate.__doc__ += BinTableHDU.tdump_file_format.replace('\n', '\n    ')
+    return BinTableHDU.load(datafile, cdfile, hfile, replace=True)
+tableload.__doc__ += BinTableHDU._tdump_file_format.replace('\n', '\n    ')
+
+
+@deprecated('3.1', alternative=':func:`tableload`')
+def tcreate(datafile, cdfile, hfile=None):
+    return tableload(datafile, cdfile, hfile)
 
 
 def _getext(filename, mode, *args, **kwargs):
@@ -667,7 +670,7 @@ def _getext(filename, mode, *args, **kwargs):
     Open the input file, return the `HDUList` and the extension.
 
     This supports several different styles of extension selection.  See the
-    `getdata()` documentation for the different possibilities.
+    :func:`getdata()` documentation for the different possibilities.
     """
 
     ext = kwargs.pop('ext', None)
@@ -738,15 +741,18 @@ def _getext(filename, mode, *args, **kwargs):
 
 def _makehdu(data, header):
     if header is None:
+        header = Header()
+    hdu = _BaseHDU(data, header)
+    if hdu.__class__ in (_BaseHDU, _ValidHDU):
+        # The HDU type was unrecognized, possibly due to a
+        # nonexistent/incomplete header
         if ((isinstance(data, np.ndarray) and data.dtype.fields is not None)
             or isinstance(data, np.recarray)):
             hdu = BinTableHDU(data)
         elif isinstance(data, np.ndarray):
             hdu = ImageHDU(data)
         else:
-            raise KeyError('Data must be numarray or table data.')
-    else:
-        hdu = _BaseHDU(data, header)
+            raise KeyError('Data must be a numpy array.')
     return hdu
 
 
