@@ -228,13 +228,21 @@ class FITSDiff(_BaseDiff):
         """
 
         if isinstance(a, basestring):
-            a = fitsopen(a)
+            try:
+                a = fitsopen(a)
+            except Exception, e:
+                raise IOError("error opening file a (%s): %s: %s" %
+                              (a, e.__class.__name__, e.args[0]))
             close_a = True
         else:
             close_a = False
 
         if isinstance(b, basestring):
-            b = fitsopen(b)
+            try:
+                b = fitsopen(b)
+            except Exception, e:
+                raise IOError("error opening file b (%s): %s: %s" %
+                              (b, e.__class.__name__, e.args[0]))
             close_b = True
         else:
             close_b = False
@@ -300,7 +308,7 @@ class FITSDiff(_BaseDiff):
         if self.ignore_comments:
             ignore_comments = ' '.join(sorted(self.ignore_comments))
             self._writeln(' Keyword(s) whose comments are not to be compared:'
-                          '\n%s' % wrapper.fill(ignore_keywords))
+                          '\n%s' % wrapper.fill(ignore_comments))
         if self.ignore_fields:
             ignore_fields = ' '.join(sorted(self.ignore_fields))
             self._writeln(' Table column(s) not to be compared:\n%s' %
@@ -1024,6 +1032,11 @@ class TableDataDiff(_BaseDiff):
             self.diff_rows = (len(self.a), len(self.b))
             return
 
+        # If the tables contain no rows there's no data to compare, so we're
+        # done at this point. (See ticket #178)
+        if len(self.a) == len(self.b) == 0:
+            return
+
         # Like in the old fitsdiff, compare tables on a column by column basis
         # The difficulty here is that, while FITS column names are meant to be
         # case-insensitive, PyFITS still allows, for the sake of flexibility,
@@ -1151,8 +1164,9 @@ def diff_values(a, b, tolerance=0.0):
     within the given relative tolerance.
     """
 
-    # TODO: Handle ifs and nans
     if isinstance(a, float) and isinstance(b, float):
+        if np.isnan(a) and np.isnan(b):
+            return False
         return not np.allclose(a, b, tolerance, 0.0)
     else:
         return a != b
