@@ -16,7 +16,7 @@ from pyfits.header import Header, HEADER_END_RE
 from pyfits.util import (lazyproperty, _is_int, _is_pseudo_unsigned,
                          _unsigned_zero, _pad_length, itersubclasses,
                          encode_ascii, decode_ascii, BLOCK_SIZE, deprecated,
-                         _get_array_memmap)
+                         _get_array_mmap)
 from pyfits.verify import _Verify, _ErrList
 
 
@@ -110,7 +110,7 @@ class _BaseHDU(object):
             elif 'CHECKSUM' in self._header:
                 self._output_checksum = True
 
-        self.name = ''
+        self._name = ''
 
     @property
     def header(self):
@@ -120,14 +120,24 @@ class _BaseHDU(object):
     def header(self, value):
         self._header = value
 
+    @property
+    def name(self):
+        return self._name
+
+    @name.setter
+    def name(self, value):
+        if not isinstance(value, basestring):
+            raise TypeError("'name' attribute must be a string")
+        self._name = value
+
     @property
     def is_image(self):
         return (
             self.name == 'PRIMARY' or
-            ('XTENSION' in self.header and
-             (self.header['XTENSION'] == 'IMAGE' or
-              (self.header['XTENSION'] == 'BINTABLE' and
-               'ZIMAGE' in self.header and self.header['ZIMAGE'] == True))))
+            ('XTENSION' in self._header and
+             (self._header['XTENSION'] == 'IMAGE' or
+              (self._header['XTENSION'] == 'BINTABLE' and
+               'ZIMAGE' in self._header and self._header['ZIMAGE'] == True))))
 
     @property
     def _data_loaded(self):
@@ -519,12 +529,12 @@ class _BaseHDU(object):
                     # Of course, if we're copying the data to a new file we
                     # don't care about flushing the original mmap; instead just
                     # read it into the new file
-                    memmap_array = None
+                    array_mmap = None
                 else:
-                    memmap_array = _get_array_memmap(self.data)
+                    array_mmap = _get_array_mmap(self.data)
 
-                if memmap_array is not None:
-                    memmap_array.flush()
+                if array_mmap is not None:
+                    array_mmap.flush()
                 else:
                     self._file.seek(self._datLoc)
                     datloc, datsize = self._writedata(fileobj)
@@ -686,6 +696,34 @@ class _ValidHDU(_BaseHDU, _Verify):
     Base class for all HDUs which are not corrupted.
     """
 
+    def __init__(self, data=None, header=None, name=None, **kwargs):
+        super(_ValidHDU, self).__init__(data=data, header=header)
+        if name is not None:
+            self.name = name
+
+        if header and not hasattr(self, '_extver'):
+            self._extver = header.get('EXTVER', 1)
+
+    @property
+    def name(self):
+        # Convert the value to a string to be flexible in some pathological
+        # cases (see ticket #96)
+        if self._header and 'EXTNAME' in self._header:
+            self._name = str(self._header['EXTNAME'])
+        return self._name
+
+    @name.setter
+    def name(self, value):
+        if not isinstance(value, basestring):
+            raise TypeError("'name' attribute must be a string")
+        if not pyfits.EXTENSION_NAME_CASE_SENSITIVE:
+            value = value.upper()
+        if self._header:
+            if 'EXTNAME' in self._header:
+                self._header['EXTNAME'] = value
+            else:
+                self._header.append(('EXTNAME', value, 'extension name'))
+
     @classmethod
     def match_header(cls, header):
         """
@@ -829,7 +867,7 @@ class _ValidHDU(_BaseHDU, _Verify):
         if 'extname' in self._header and savecomment:
             comment = None
 
-        self._header.set('extname', value, comment, before, after)
+        self._header.set('EXTNAME', value, comment, before, after)
         self.name = value
 
     def update_ext_version(self, value, comment=None, before=None,
@@ -921,6 +959,19 @@ class _ValidHDU(_BaseHDU, _Verify):
                             self.run_option(option=option, err_text=err_text,
                                             fix=fix, fix_text="Deleted."))
 
+
+        # Verify that the EXTNAME keyword exists and is a string
+        if 'EXTNAME' in self._header:
+            if not isinstance(self._header['EXTNAME'], basestring):
+                err_text = 'The EXTNAME keyword must have a string value.'
+                fix_text = 'Converted the EXTNAME keyword to a string value.'
+
+                def fix(header=self._header):
+                    header['EXTNAME'] = str(header['EXTNAME'])
+
+                errs.append(self.run_option(option, err_text=err_text,
+                                            fix_text=fix_text, fix=fix))
+
         # verify each card
         for card in self._header.cards:
             errs.append(card._verify(option))
@@ -1399,38 +1450,6 @@ class ExtensionHDU(_ValidHDU):
 
     _extension = ''
 
-    def __init__(self, data=None, header=None, name=None, **kwargs):
-        super(ExtensionHDU, self).__init__(data=data, header=header)
-        if header:
-            if name is None:
-                if not self.name and 'EXTNAME' in header:
-                    self.name = header['EXTNAME']
-            else:
-                self.name = name
-
-            if not hasattr(self, '_extver'):
-                if 'EXTVER' in header:
-                    self._extver = header['EXTVER']
-                else:
-                    self._extver = 1
-
-    def __setattr__(self, attr, value):
-        """
-        Set an HDU attribute.
-        """
-
-        if attr == 'name' and value:
-            if not isinstance(value, basestring):
-                raise TypeError("'name' attribute must be a string")
-            if not pyfits.EXTENSION_NAME_CASE_SENSITIVE:
-                value = value.upper()
-            if 'EXTNAME' in self._header:
-                self._header['EXTNAME'] = value
-            else:
-                self._header.append(('EXTNAME', value, 'extension name'))
-
-        super(ExtensionHDU, self).__setattr__(attr, value)
-
     @classmethod
     def match_header(cls, header):
         """
@@ -1466,6 +1485,7 @@ class ExtensionHDU(_ValidHDU):
                        0, option, errs)
         self.req_cards('GCOUNT', naxis + 4, lambda v: (_is_int(v) and v == 1),
                        1, option, errs)
+
         return errs
 # For backwards compatilibity, though this needs to be deprecated
 # TODO: Mark this as deprecated
