@@ -1,6 +1,5 @@
 from __future__ import division, with_statement  # confidence high
 
-import shutil
 import warnings
 
 import numpy as np
@@ -215,15 +214,15 @@ class TestChecksumFunctions(PyfitsTestCase):
         to the file.
         """
 
-        shutil.copy(self.data('checksum.fits'), self.temp('tmp.fits'))
+        self.copy_file('checksum.fits')
 
-        with pyfits.open(self.temp('tmp.fits')) as hdul:
+        with pyfits.open(self.temp('checksum.fits')) as hdul:
             data = hdul[1].data.copy()
 
-        hdul = pyfits.open(self.temp('tmp.fits'), mode='update')
+        hdul = pyfits.open(self.temp('checksum.fits'), mode='update')
         hdul.close()
 
-        with pyfits.open(self.temp('tmp.fits')) as hdul:
+        with pyfits.open(self.temp('checksum.fits')) as hdul:
             assert_true('CHECKSUM' in hdul[1].header)
             assert_true('DATASUM' in hdul[1].header)
             assert_true((data == hdul[1].data).all())
@@ -238,16 +237,16 @@ class TestChecksumFunctions(PyfitsTestCase):
         opened with checksum='remove'.
         """
 
-        shutil.copy(self.data('checksum.fits'), self.temp('tmp.fits'))
-        with pyfits.open(self.temp('tmp.fits')) as hdul:
+        self.copy_file('checksum.fits')
+        with pyfits.open(self.temp('checksum.fits')) as hdul:
             header = hdul[1].header.copy()
             data = hdul[1].data.copy()
 
-        with pyfits.open(self.temp('tmp.fits'), mode='update') as hdul:
+        with pyfits.open(self.temp('checksum.fits'), mode='update') as hdul:
             hdul[1].header['FOO'] = 'BAR'
             hdul[1].data[0]['TIME'] = 42
 
-        with pyfits.open(self.temp('tmp.fits')) as hdul:
+        with pyfits.open(self.temp('checksum.fits')) as hdul:
             header2 = hdul[1].header
             data2 = hdul[1].data
             assert_equal(header2[:-3], header[:-2])
@@ -257,11 +256,11 @@ class TestChecksumFunctions(PyfitsTestCase):
             assert_true((data2['TIME'][1:] == data['TIME'][1:]).all())
             assert_equal(data2['TIME'][0], 42)
 
-        with pyfits.open(self.temp('tmp.fits'), mode='update',
+        with pyfits.open(self.temp('checksum.fits'), mode='update',
                          checksum='remove') as hdul:
             pass
 
-        with pyfits.open(self.temp('tmp.fits')) as hdul:
+        with pyfits.open(self.temp('checksum.fits')) as hdul:
             header2 = hdul[1].header
             data2 = hdul[1].data
             assert_equal(header2[:-1], header[:-2])
