@@ -295,7 +295,6 @@ class TestHeaderFunctions(PyfitsTestCase):
             warnings.simplefilter('error')
             assert_raises(UserWarning, pyfits.Card, 'abcdefghi', 'long')
 
-
     def test_illegal_characters_in_key(self):
         """
         Test that Card constructor allows illegal characters in the keyword,
@@ -447,6 +446,24 @@ class TestHeaderFunctions(PyfitsTestCase):
             "CONTINUE  '&' / comment long comment long comment long comment long comment     "
             "CONTINUE  '&' / long comment                                                    ")
 
+    def test_long_unicode_string(self):
+        """Regression test for
+        https://github.com/spacetelescope/PyFITS/issues/1
+
+        So long as a unicode string can be converted to ASCII it should have no
+        different behavior in this regard from a byte string.
+        """
+
+        h1 = pyfits.Header()
+        h1['TEST'] = 'abcdefg' * 30
+
+        h2 = pyfits.Header()
+        with catch_warnings(record=True) as w:
+            h2['TEST'] = u'abcdefg' * 30
+            assert_equal(len(w), 0)
+
+        assert_equal(str(h1), str(h2))
+
     def test_long_string_repr(self):
         """Regression test for #193
 
@@ -567,6 +584,68 @@ class TestHeaderFunctions(PyfitsTestCase):
                      "HIERARCH ESO INS SLIT2 Y1FRML= "
                      "'ENC=OFFSET+RESOL*acos((WID-(MAX+MIN))/(MAX-MIN)'")
 
+    def test_hierarch_with_abbrev_value_indicator(self):
+        """Regression test for
+        https://github.com/spacetelescope/PyFITS/issues/5
+        """
+
+        c = pyfits.Card.fromstring("HIERARCH key.META_4='calFileVersion'")
+        assert_equal(c.keyword, 'key.META_4')
+        assert_equal(c.value, 'calFileVersion')
+        assert_equal(c.comment, '')
+
+    def test_hierarch_keyword_whitespace(self):
+        """
+        Regression test for
+        https://github.com/spacetelescope/PyFITS/issues/6
+
+        Make sure any leading or trailing whitespace around HIERARCH
+        keywords is stripped from the actual keyword value.
+        """
+
+        c = pyfits.Card.fromstring(
+                "HIERARCH  key.META_4    = 'calFileVersion'")
+        assert_equal(c.keyword, 'key.META_4')
+        assert_equal(c.value, 'calFileVersion')
+        assert_equal(c.comment, '')
+
+        # Test also with creation via the Card constructor
+        c = pyfits.Card('HIERARCH  key.META_4', 'calFileVersion')
+        assert_equal(c.keyword, 'key.META_4')
+        assert_equal(c.value, 'calFileVersion')
+        assert_equal(c.comment, '')
+
+    def test_verify_mixed_case_hierarch(self):
+        """Regression test for
+        https://github.com/spacetelescope/PyFITS/issues/7
+
+        Assures that HIERARCH keywords with lower-case characters and other
+        normally invalid keyword characters are not considered invalid.
+        """
+
+        c = pyfits.Card('HIERARCH WeirdCard.~!@#_^$%&', 'The value',
+                        'a comment')
+        # This should not raise any exceptions
+        c.verify('exception')
+        assert_equal(c.keyword, 'WeirdCard.~!@#_^$%&')
+        assert_equal(c.value, 'The value')
+        assert_equal(c.comment, 'a comment')
+
+        # Test also the specific case from the original bug report
+        header = pyfits.Header([
+            ('simple', True),
+            ('BITPIX', 8),
+            ('NAXIS', 0),
+            ('EXTEND', True, 'May contain datasets'),
+            ('HIERARCH key.META_0', 'detRow')
+        ])
+        hdu = pyfits.PrimaryHDU(header=header)
+        hdu.writeto(self.temp('test.fits'))
+        with pyfits.open(self.temp('test.fits')) as hdul:
+            header2 = hdul[0].header
+            assert_equal(str(header.cards[header.index('key.META_0')]),
+                         str(header2.cards[header2.index('key.META_0')]))
+
     def test_missing_keyword(self):
         """Test that accessing a non-existent keyword raises a KeyError."""
 
