@@ -3,7 +3,6 @@ from __future__ import with_statement
 
 import math
 import os
-import shutil
 import time
 import warnings
 
@@ -15,6 +14,7 @@ from pyfits.tests.util import catch_warnings, ignore_warnings, CaptureStdio
 
 from nose.tools import (assert_equal, assert_raises, assert_true, assert_false,
                         assert_not_equal)
+from numpy.testing import assert_allclose
 
 
 class TestImageFunctions(PyfitsTestCase):
@@ -619,7 +619,7 @@ class TestImageFunctions(PyfitsTestCase):
         """
 
         # Copy the original file before making any possible changes to it
-        shutil.copy(self.data('comp.fits'), self.temp('comp.fits'))
+        self.copy_file('comp.fits')
         mtime = os.stat(self.temp('comp.fits')).st_mtime
 
         time.sleep(1)
@@ -774,14 +774,14 @@ class TestImageFunctions(PyfitsTestCase):
         """
 
         # Copy the original file before saving to it
-        shutil.copy(self.data('test0.fits'), self.temp('test_new.fits'))
-        with pyfits.open(self.temp('test_new.fits'), mode='update') as hdul:
+        self.copy_file('test0.fits')
+        with pyfits.open(self.temp('test0.fits'), mode='update') as hdul:
             orig_data = hdul[1].data.copy()
             hdr_copy = hdul[1].header.copy()
             del hdr_copy['NAXIS*']
             hdul[1].header = hdr_copy
 
-        with pyfits.open(self.temp('test_new.fits')) as hdul:
+        with pyfits.open(self.temp('test0.fits')) as hdul:
             assert_true((orig_data == hdul[1].data).all())
 
     def test_open_scaled_in_update_mode(self):
@@ -795,7 +795,7 @@ class TestImageFunctions(PyfitsTestCase):
         """
 
         # Copy the original file before making any possible changes to it
-        shutil.copy(self.data('scale.fits'), self.temp('scale.fits'))
+        self.copy_file('scale.fits')
         mtime = os.stat(self.temp('scale.fits')).st_mtime
 
         time.sleep(1)
@@ -890,7 +890,7 @@ class TestImageFunctions(PyfitsTestCase):
     def test_scale_back(self):
         """A simple test for #120--the scale_back feature for image HDUs."""
 
-        shutil.copy(self.data('scale.fits'), self.temp('scale.fits'))
+        self.copy_file('scale.fits')
         with pyfits.open(self.temp('scale.fits'), mode='update',
                          scale_back=True) as hdul:
             orig_bitpix = hdul[0].header['BITPIX']
@@ -951,3 +951,25 @@ class TestImageFunctions(PyfitsTestCase):
                 # above
                 hdul2[0].data[0] = 0
                 assert_true((hdul[1].data == hdul2[0].data).all())
+
+    def test_compression_column_tforms(self):
+        """Regression test for #199."""
+
+        # Some interestingly tiled data so that some of it is quantized and
+        # some of it ends up just getting gzip-compressed
+        data2 = ((np.arange(1, 8, dtype=np.float32) * 10)[:, np.newaxis] +
+                np.arange(1, 7))
+        np.random.seed(1337)
+        data1 = np.random.uniform(size=(6 * 4, 7 * 4))
+        data1[:data2.shape[0], :data2.shape[1]] = data2
+        chdu = pyfits.CompImageHDU(data1, compressionType='RICE_1',
+                                   tileSize=(6, 7))
+        chdu.writeto(self.temp('test.fits'))
+
+        with pyfits.open(self.temp('test.fits'),
+                         disable_image_compression=True) as h:
+            assert_equal(h[1].header['TFORM1'], '1PB(51)')
+            assert_equal(h[1].header['TFORM2'], '1PD(2)')
+
+        with pyfits.open(self.temp('test.fits')) as h:
+            assert_allclose(h[1].data, data1, rtol=0.1, atol=0.1)
