@@ -1,4 +1,4 @@
-from __future__ import division
+from __future__ import division, with_statement
 
 import platform
 
@@ -61,3 +61,21 @@ class TestUintFunctions(PyfitsTestCase):
             assert_true((hdul[0].section[:1] == hdul[0].data[:1]).all())
             hdul.close()
             hdul1.close()
+
+    def test_uint_compressed(self):
+        hdu = pyfits.CompImageHDU(np.array([-3, -2, -1, 0, 1, 2, 3]))
+        hdu.scale('int32', '', bzero=2**31)
+        hdu.writeto(self.temp('temp.fits'))
+        with pyfits.open(self.temp('temp.fits'), uint=True) as hdul:
+            assert_equal(hdul[1].data.dtype, np.uint32)
+            assert_true(
+                (hdul[1].data ==
+                 np.array([(2**32)-3, (2**32)-2, (2**32)-1, 0, 1, 2, 3],
+                          dtype=np.uint32)).all())
+            hdul.writeto(self.temp('temp2.fits'))
+            with pyfits.open(self.temp('temp2.fits'), uint=True) as hdul2:
+                assert_true((hdul[1].data == hdul2[1].data).all())
+                # TODO: Enable these lines if CompImageHDUs ever grow .section
+                # support
+                #assert_equal(hdul[1].section[:1].dtype.name, 'uint32')
+                #assert_true((hdul[1].section[:1] == hdul[1].data[:1]).all())
