@@ -1,6 +1,7 @@
 import __builtin__
 import functools
 import itertools
+import mmap
 import os
 import signal
 import sys
@@ -24,8 +25,6 @@ except ImportError:
 
 import numpy as np
 
-from numpy import memmap as Memmap
-
 
 BLOCK_SIZE = 2880  # the FITS block size
 
@@ -589,13 +588,17 @@ def _write_string(f, s):
 def _convert_array(array, dtype):
     """
     Converts an array to a new dtype--if the itemsize of the new dtype is
-    the same as the old dtype, a view is returned.  Otherwise a new array must
-    be created.
+    the same as the old dtype and both types are not numeric, a view is
+    returned.  Otherwise a new array must be created.
     """
 
     if array.dtype == dtype:
         return array
-    elif array.dtype.itemsize == dtype.itemsize:
+    elif (array.dtype.itemsize == dtype.itemsize and not
+            (np.issubdtype(array.dtype, np.number) and
+             np.issubdtype(dtype, np.number))):
+        # Includes a special case when both dtypes are at least numeric to
+        # account for ticket #218: https://trac.assembla.com/pyfits/ticket/218
         return array.view(dtype)
     else:
         return array.astype(dtype)
@@ -734,18 +737,30 @@ def _tmp_name(input):
     return fn
 
 
-def _get_array_memmap(array):
+if sys.version_info[:2] < (2, 6):
+    # In Python 2.5 mmap.mmap is a function that returns an object of type
+    # 'mmap.mmap', but the mmap.mmap type is otherwise not accessible through
+    # the module
+    def _is_mmap(obj):
+        return (type(obj).__module__ == 'mmap' and
+                type(obj).__name__ == 'mmap')
+else:
+    def _is_mmap(obj):
+        return isinstance(obj, mmap.mmap)
+
+
+def _get_array_mmap(array):
     """
-    If the array has a numpy.memmap as one of its bases, return the memmap
-    base; otherwise return None.
+    If the array has an mmap.mmap at base of its base chain, return the mmap
+    object; otherwise return None.
     """
 
-    if isinstance(array, Memmap):
+    if _is_mmap(array):
         return array
 
     base = array
     while hasattr(base, 'base') and base.base is not None:
-        if isinstance(base.base, Memmap):
+        if _is_mmap(base.base):
             return base.base
         base = base.base
 
